#!/bin/bash
#
# Simple toolkit to manage websites under Apache-2 webserver on Debian.
# Part of website-manager-2 package. Sorry about my terrible English! :-)

# Including headers
HEADERS="/etc/default/wsm2"
MSG_WSM_NOHEADER="Header file $HEADERS is not found - terminating."
if [ ! -r "$HEADERS" ]; then echo -e "$MSG_WSM_NOHEADER" >&2; exit 1; fi
. $HEADERS

#
# Functions
#

# wsm_createweb website [email [webmaster [wmpassword [auditor [aupassword]]]]
# Creates a static website with DAV management and web statistics generation.

function wsm_createweb {

# Checking components and circumstances.
    if [ ! $UID -eq 0 ]; then log $MSG_WSM_ALL_ROOTNEED; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMHTTPTEMPLATE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMHTTPTEMPLATE" >&2; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMHTMLPAGE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMHTMLPAGE" >&2; return $EXIT_ERR; fi

    # Website is mandatory and formally checked.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log "$MSG_WSM_CREATEWEB_NOWEBSITE"; return $EXIT_ERR; fi

    # Webmaster's email is optional, but formally checked.
    local email="$1"; shift
    if [ -z "$email" ]; then email=$DEFAULT_EMAIL; fi
    if [ ! -z "`echo "$email" | $SED "s/$MASKEMAIL//"`" ]; then log "$MSG_WSM_CREATEWEB_WRONGEMAIL"; return $EXIT_ERR; fi

    # Webmaster's username is optional, but formally checked.
    local webmaster="$1"; shift
    if [ -z "$webmaster" ]; then webmaster=$DEFAULT_WEBMASTER; fi
    if [ ! -z "`echo "$webmaster" | $SED "s/$MASKUSER//"`" ]; then log "$MSG_WSM_CREATEWEB_WRONGWMUSER"; return $EXIT_ERR; fi

    # Webmaster's password is optional, but formally checked.
    local wmpassword="$1"; shift
    if [ -z "$wmpassword" ]; then wmpassword=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$wmpassword" | $SED "s/$MASKPASS//"`" ]; then log "$MSG_WSM_CREATEWEB_WRONGWMPWD"; return $EXIT_ERR; fi

    # Auditor's username is optional, but formally checked.
    local auditor="$1"; shift
    if [ -z "$auditor" ]; then auditor=$DEFAULT_AUDITOR; fi
    if [ ! -z "`echo "$auditor" | $SED "s/$MASKUSER//"`" ]; then log "$MSG_WSM_CREATEWEB_WRONGAUUSER"; return $EXIT_ERR; fi

    # Auditor's password is optional, but formally checked.
    local aupassword="$1"; shift
    if [ -z "$aupassword" ]; then aupassword=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$aupassword" | $SED "s/$MASKPASS//"`" ]; then log "$MSG_WSM_CREATEWEB_WRONGAUPWD"; return $EXIT_ERR; fi

    # Avoid overwriting any existing content.
    if [ -d "$APACHEDOCROOT/$vhost" ]; then log "$MSG_WSM_CREATEWEB_DOCROOTEXIST"; return $EXIT_ERR; fi
    if [ -f "$APACHEVHOSTSDIR/$vhost" ]; then log "$MSG_WSM_CREATEWEB_HTTPCONFEXIST"; return $EXIT_ERR; fi
    if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then log "$MSG_WSM_CREATEWEB_HTTPUSEREXIST"; return $EXIT_ERR; fi
    if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then log "$MSG_WSM_CREATEWEB_HTTPGROUPEXIST"; return $EXIT_ERR; fi

    local TIMESTAMP=$(date '+%Y-%m-%d %H:%M:%S')

# Creating web users and groups.
    wsm_adduser $vhost $webmaster $wmpassword
    wsm_addgroup $vhost $APACHEVHADMINGROUP
    wsm_addusertogroup $vhost $webmaster $APACHEVHADMINGROUP
    wsm_adduser $vhost $auditor $aupassword
    wsm_addgroup $vhost $APACHEVHSTATSGROUP
    wsm_addusertogroup $vhost $auditor $APACHEVHSTATSGROUP

# Creating web document root and minimal content: 
# indexpage, robots file, download, binary, config, download, log, upload directory 
# and stats link.
    # Document root.
    if [ ! -d $APACHEDOCROOT/$vhost ]; then
        mkdir -m 2750 $APACHEDOCROOT/$vhost
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost
        log "$APACHEDOCROOT/$vhost" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Index page.
    if [ ! -f $APACHEDOCROOT/$vhost/$DEFAULT_PAGE ]; then
        export TIMESTAMP vhost
        cat $WSMTEMPLATEDIR/$WSMHTMLPAGE | $ENVSUBST > $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
        chmod 640 $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
        log "$APACHEDOCROOT/$vhost/$DEFAULT_PAGE" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$DEFAULT_PAGE" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Robots page.
    if [ ! -f $APACHEDOCROOT/$vhost/$ROBOTS_PAGE ]; then
        export TIMESTAMP vhost
        cat $WSMTEMPLATEDIR/$WSMBOTSFILE | $ENVSUBST > $APACHEDOCROOT/$vhost/$ROBOTS_PAGE
        chmod 640 $APACHEDOCROOT/$vhost/$ROBOTS_PAGE
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$ROBOTS_PAGE
        log "$APACHEDOCROOT/$vhost/$ROBOTS_PAGE" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$ROBOTS_PAGE" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Webcron file.
    if [ ! -f $APACHEDOCROOT/$vhost/$WEBCRONTAB ]; then
        export TIMESTAMP vhost
        cat $WSMTEMPLATEDIR/$WEBCRONTAB | $ENVSUBST > $APACHEDOCROOT/$vhost/$WEBCRONTAB
        chmod 640 $APACHEDOCROOT/$vhost/$WEBCRONTAB
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$WEBCRONTAB
        log "$APACHEDOCROOT/$vhost/$WEBCRONTAB" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$WEBCRONTAB" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Download directory
    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR ]; then
        mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR
        log "$APACHEDOCROOT/$vhost/$APACHEDNLOADDIR" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$APACHEDNLOADDIR" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Upload directory writable by webserver.
    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR ]; then
        mkdir -m 2770 $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
        if [ -x $SETFACL ]; then
            # Force group rw inheritable below this folder.
            $SETFACL -d -m g::rwX $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
        fi
        log "$APACHEDOCROOT/$vhost/$APACHEUPLOADDIR" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$APACHEUPLOADDIR" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Directory for symlinked binaries.
    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEVHBINDIR ]; then
        mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEVHBINDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHBINDIR
        log "$APACHEDOCROOT/$vhost/$APACHEVHBINDIR" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$APACHEVHBINDIR" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Directory for configuration files.
    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR ]; then
        mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR
        log "$APACHEDOCROOT/$vhost/$APACHEVHCONFDIR" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$APACHEVHCONFDIR" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Directory for symlinked logfiles.
    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR ]; then
        mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR
        log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR" "$MSG_WSM_CREATEWEB_CREATED"
    else
        log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR" "$MSG_WSM_CREATEWEB_EXISTS"
    fi
    for logfile in $APACHELOGS
    do
        if [ ! -L $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile ]; then
            ln -s $APACHELOGDIR/$vhost-$logfile $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile
            chown -h $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile
            log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile -> $APACHELOGDIR/$vhost-$logfile"
        else
            log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile $MSG_WSM_CREATEWEB_EXISTS"
        fi
    done
    for logfile in $PHPLOGS
    do
        if [ ! -L $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile ]; then
            ln -s $PHPLOGDIR/$vhost-$logfile $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile
            chown -h $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile
            log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile -> $PHPLOGDIR/$vhost-$logfile"
        else
            log "$APACHEDOCROOT/$vhost/$APACHEVHLOGDIR/$logfile $MSG_WSM_CREATEWEB_EXISTS"
        fi
    done
    # Prepared symlink for AwStats
    if [ ! -L $APACHEDOCROOT/$vhost/$AWSTATSLINK ]; then
        ln -s $AWSTATSWEBROOT/$vhost $APACHEDOCROOT/$vhost/$AWSTATSLINK
        chown -h $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$AWSTATSLINK
        log "$APACHEDOCROOT/$vhost/$AWSTATSLINK -> $AWSTATSWEBROOT/$vhost"
    else
        log "$APACHEDOCROOT/$vhost/$AWSTATSLINK $MSG_WSM_CREATEWEB_EXISTS"
    fi

# Apache webserver configuration.
    if [ ! -f $APACHEVHOSTDDIR/$vhost ]; then
        export  APACHEADMIN APACHEADMINGROUP APACHEUSER APACHEGROUP \
                APACHEVHADMINGROUP APACHEVHSTATSGROUP \
                APACHEDOCROOT APACHEAUTHDIR APACHELOGDIR \
                APACHEVHBINDIR APACHEVHCONFDIR APACHEDNLOADDIR APACHEUPLOADDIR \
                APACHESSLIP APACHESSLPORT APACHEWEBIP APACHEWEBPORT \
                AUDITLOGGER DEFAULT_HOSTNAME PHPLOGDIR TIMESTAMP email vhost
        cat $WSMTEMPLATEDIR/$WSMHTTPTEMPLATE | $ENVSUBST > $APACHEVHOSTSDIR/$vhost
        chown root:root $APACHEVHOSTSDIR/$vhost; chmod 644 $APACHEVHOSTSDIR/$vhost
    else
        log "$APACHECONFIGDIR/$vhost $MSG_WSM_CREATEWEB_EXISTS"
    fi
    # Activating new virtualhost.
    log "$($APACHEENSITE $vhost)"
    log "$($APACHEDAEMON reload)"

# That's all, folks :-)
    return $EXIT_SUCC
}

# wsm_saveweb website [--noapache] [--noawstats] [--nobins] [--nodocs] [--nologs]
# Save a website - webpages, users, logs, configurations, statistics.

function wsm_saveweb {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log "$MSG_WSM_ALL_NOWEBSITE"; return $EXIT_ERR; fi

# Parsing optional options.
    local saveapache="Y"
    local saveawstats="Y"
    local savebins="Y"
    local savedocs="Y"
    local savelogs="Y"
    local option="$1"; shift
    while [ ${#option} -gt 0 ];
    do
        case $option in
            --noapache)
                saveapache="N"
                ;;
            --noawstats)
                saveawstats="N"
                ;;
            --nobins)
                savebins="N"
                ;;
            --nodocs)
                savedocs="N"
                ;;
            --nologs)
                savelogs="N"
                ;;
        esac
        option="$1"; shift
    done

    local backupfile=""
    local datetime="`$DATE '+%Y%m%d%H%M%S'`"
    local taroptions=""

# Web content from docroot save.
    if [ $savedocs = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.docroot.tar.gz"
        if [ -d $APACHEDOCROOT/$vhost ]; then
            ( cd  $APACHEDOCROOT/$vhost ;
              $TAR 'czf' $backupfile .[^.]* *
            )
            wait
        else
            log "$MSG_WSM_SAVEWEB_NODOCROOT $APACHEDOCROOT/$vhost"
        fi
    fi

# Binaries from binary path save.
    if [ $savebins = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.binary.tar.gz"
        if [ -d $APACHEBINROOT/$vhost ]; then
            ( cd  $APACHEBINROOT/$vhost ;
              $TAR 'czf' $backupfile .[^.]* *
            )
            wait
        else
            log "$MSG_WSM_SAVEWEB_NOBINROOT $APACHEBINROOT/$vhost"
        fi
    fi

# Apache configuration, users and groups.
    if [ $saveapache = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.apache.tar"
        if [ -f "$APACHEVHOSTSDIR/$vhost" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $APACHEVHOSTSDIR/$vhost 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEVHOSTSDIR/$vhost"
        fi
        if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $APACHEAUTHDIR/$vhost.group 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.group"
        fi
        if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $APACHEAUTHDIR/$vhost.user 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.user"
        fi
        if [ -f $backupfile ]; then $GZIP $backupfile; fi
    fi

# Logs.
    if [ $savelogs = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.apachelogs.tar.gz"
# Apache logs.
        ( cd $APACHELOGDIR ;
          if [ ! -z "`ls $vhost* 2>/dev/null`" ]; then
              $TAR 'czf' $backupfile $vhost*
          else
              log "$MSG_WSM_SAVEWEB_NOAPACHELOG $APACHELOGDIR/$vhost*"
          fi
        )
        wait
# Optional PHP logs.
        backupfile="`pwd`/`hostname`.$vhost.$datetime.phplogs.tar.gz"
        if [ -d "$PHPLOGDIR" ]; then
            ( cd $PHPLOGDIR ;
              if [ ! -z "`ls $vhost* 2>/dev/null`" ]; then
                  $TAR 'czf' $backupfile $vhost*
              else
                  log "$MSG_WSM_SAVEWEB_NOPHPLOG $PHPLOGDIR/$vhost*"
              fi
            )
        fi
        wait
    fi

# AwStats configuration, results and webpages
    if [ $saveawstats = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.awstats.tar"
        if [ -f "$AWSTATSCONFIGDIR/awstats.$vhost.conf" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $AWSTATSCONFIGDIR/awstats.$vhost.conf 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSCONFIGDIR/awstats.$vhost.conf"
        fi
        if [ ! -z "`ls $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null`" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSCONTDIR/*.$vhost.*"
        fi
        if [ -d "$AWSTATSWEBROOT/$vhost" ]; then
            if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
            $TAR $taroptions'f' $backupfile $AWSTATSWEBROOT/$vhost 2>/dev/null
        else
            log "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSWEBROOT/$vhost/*"
        fi
        if [ -f $backupfile ]; then $GZIP $backupfile; fi
    fi
}

# wsm_removeweb website [force]
# Remove a website - webpages, binaries, users, logs, configurations, statistics.

function wsm_removeweb {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log "$MSG_WSM_ALL_NOWEBSITE"; return $EXIT_ERR; fi

# Confirmation request if not forced.
    if [ "$1" != "force" ]; then
        read -s -n1 -p "$MSG_WSM_RMVWEB_CONFIRM"; echo
        if [ "$REPLY" != "y" -a "$REPLY" != "Y" ]; then return $EXIT_ERR; fi
    fi

# Deactivating virtualhost.
    log "$($APACHEDISSITE $vhost)"
    log "$($APACHEDAEMON reload)"

# Removing Apache configuration, users and groups.
    if [ -f "$APACHEVHOSTSDIR/$vhost" ]; then
        rm -f $APACHEVHOSTSDIR/$vhost 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $APACHEVHOSTSDIR/$vhost"
    else
        log "$MSG_WSM_RMVWEB_NOAPACHE $APACHEVHOSTSDIR/$vhost"
    fi
    if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then
        rm -f $APACHEAUTHDIR/$vhost.group 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $APACHEAUTHDIR/$vhost.group"
    else
        log "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.group"
    fi
    if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then
        rm -f $APACHEAUTHDIR/$vhost.user 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $APACHEAUTHDIR/$vhost.user"
    else
        log  "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.user"
    fi

# Removing Apache logs.
    if [ ! -z "`ls $APACHELOGDIR/$vhost* 2>/dev/null`" ]; then
        rm -f $APACHELOGDIR/$vhost* 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $APACHELOGDIR/$vhost*"
    else
        log "$MSG_WSM_RMVWEB_NOAPACHELOG $APACHELOGDIR/$vhost*"
    fi

# Removing PHP logs.
    if [ ! -z "`ls $PHPLOGDIR/$vhost* 2>/dev/null`" ]; then
        rm -f $PHPLOGDIR/$vhost* 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $PHPLOGDIR/$vhost*"
    else
        log  "$MSG_WSM_RMVWEB_NOPHPLOG $PHPLOGDIR/$vhost*"
    fi

# Removing AwStats configuration, results and webpages
    if [ -f "$AWSTATSCONFIGDIR/awstats.$vhost.conf" ]; then
        rm -f $AWSTATSCONFIGDIR/awstats.$vhost.conf 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $AWSTATSCONFIGDIR/awstats.$vhost.conf"
    else
        log "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSCONFIGDIR/awstats.$vhost.conf" >&2
    fi
    if [ ! -z "`ls $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null`" ]; then
        rm -f $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $AWSTATSCONTDIR/*.$vhost.*"
    else
        log "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSCONTDIR/*.$vhost.*"
    fi
    if [ -d "$AWSTATSWEBROOT/$vhost" ]; then
        rm -Rf $AWSTATSWEBROOT/$vhost 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $AWSTATSWEBROOT/$vhost"
    else
        log "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSWEBROOT/$vhost/*"
    fi

# Remove web content from docroot.
    if [ -d $APACHEDOCROOT/$vhost ]; then
        rm -Rf $APACHEDOCROOT/$vhost 2>/dev/null
        log "$MSG_WSM_RMVWEB_REMOVED $APACHEDOCROOT/$vhost"
    else
        log  "$MSG_WSM_RMVWEB_NODOCROOT $APACHEDOCROOT/$vhost"
    fi

# Remove binaries from binary store.
    if [ -d $APACHEBINROOT/$vhost ]; then
        rm -Rf $APACHEBINROOT/$vhost 2>/dev/null
        if [ ! -d $APACHEBINROOT/$vhost ]; then
            log "$MSG_WSM_RMVWEB_REMOVED $APACHEBINROOT/$vhost"
        else
            log "$MSG_WSM_RMVWEB_NOTREMOVED $APACHEBINROOT/$vhost"
        fi
    else
        log "$MSG_WSM_RMVWEB_NOBINROOT $APACHEBINROOT/$vhost"
    fi

    return $EXIT_SUCC
}

# wsm_adduser website username [password]
# Adds a user to website's htpasswd file. If already exist, changes password.
# Finally echoes website, username and password used.

function wsm_adduser {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# User name is mandatory and formally checked.
    local username="$1"; shift
    if [ -z "$username" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi
    if [ ! -z "`echo "$username" | $SED "s/$MASKUSER//"`" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi

# Password is optional, if empty, random password used.
# Password is formally checked also.
    local password="$1"; shift
    if [ -z "$password" ]; then password=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$password" | $SED "s/$MASKPASS//"`" ]; then log "$MSG_WSM_ADDUSER_WRONGPWD"; return $EXIT_ERR; fi

# Creates user file, if it's not found, then calls htpasswd.
    $TOUCH $APACHEAUTHDIR/$vhost.user
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.user
    chmod 440 $APACHEAUTHDIR/$vhost.user
    $HTPASSWD -b $APACHEAUTHDIR/$vhost.user $username $password 2>/dev/null
    log $MSG_WSM_ADDUSER_ADDED $vhost $username $password
    return $EXIT_SUCC
}

# wsm_deluser website username
# Deletes given user from website's htpasswd file.
# Deletes group memeberships also, using 
# wsm_listusersgroup and wsm_deluserfromgroup functions.

function wsm_deluser {

# Website is mandatory and must exist.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi
    if [ ! -f "$APACHEAUTHDIR/$vhost.user" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi

# Removing group memberships.
    local membership=`wsm_listusersgroup $vhost $username`
    local groupname=""
    for groupname in $membership
    do
        wsm_deluserfromgroup $vhost $username $groupname
    done

# Calls htpasswd with -D parameter.
    $HTPASSWD -D $APACHEAUTHDIR/$vhost.user $username 2>/dev/null
    log $MSG_WSM_DELUSER_DELETED $vhost $username
    return $EXIT_SUCC
}

# wsm_addgroup website groupname
# Adds a user's group for website.

function wsm_addgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# Group name is mandatory and formally checked.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi
    if [ ! -z "`echo "$groupname" | $SED "s/$MASKGROUP//"`" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi

# Creates group file and group line, if it's not found.
    $TOUCH $APACHEAUTHDIR/$vhost.group
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then 
        echo "$groupname:" >> $APACHEAUTHDIR/$vhost.group
        log $MSG_WSM_ADDGROUP_ADDED $vhost $groupname
    else
        log $MSG_WSM_ADDGROUP_EXISTS $vhost $groupname
    fi
    return $EXIT_SUCC
}

# wsm_delgroup website groupname
# Delete a group line from website's group file.

function wsm_delgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi

# Delete group line.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | $GREP -v "$groupname:" >> $APACHEAUTHDIR/$vhost.group
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    log $MSG_WSM_DELGROUP_DELETED $vhost $groupname
    return $EXIT_SUCC
}

# wsm_listusersgroup website username
# Lists all groups containing given user.

function wsm_listusersgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi

# Enumerates group lines and finds username.
    cat $APACHEAUTHDIR/$vhost.group 2>/dev/null | \
    $SED "s/\(:\|,\)/ /g" | \
    $SED -n "s/^\($MASKGROUP\)\(.*\)\($username\)\(.*\)$/\1/p"
    return $EXIT_SUCC
}

# wsm_addusertogroup website user group
# Adds a user to a group of website.

function wsm_addusertogroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi

# Gets group line then add user if isn't found.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | while read line
    do
        if [ ! -z "`echo $line | $SED -n "s/^\($groupname:\)\(.*\)$/\1/p"`" ]; then
            if [ -z "`echo $line | $SED -n "s/^\($groupname\)\(.*\)\(:\|,\)$username\(,\|$\)\(.*\)$/\1/p"`" ]; then
                if [ -z "`echo $line | $SED -n "s/^\(.*\):$/\1/p"`" ]; then
                    echo "$line $username" >> $APACHEAUTHDIR/$vhost.group
                else
                    echo "$line$username" >> $APACHEAUTHDIR/$vhost.group
                fi
                log $MSG_WSM_ADDUSERTOGROUP_ADDED $vhost $username $groupname
            else
                echo $line >> $APACHEAUTHDIR/$vhost.group
                log $MSG_WSM_ADDUSERTOGROUP_EXISTS $vhost $username $groupname
            fi
        else
            echo $line >> $APACHEAUTHDIR/$vhost.group
        fi
    done
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    return $EXIT_SUCC
}

# wsm_deluserfromgroup website user group
# Delete given website user from the given group.

function wsm_deluserfromgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then log $MSG_WSM_ALL_NOWEBSITE; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then log $MSG_WSM_ALL_NOUSER; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then log $MSG_WSM_ALL_NOGROUP; return $EXIT_ERR; fi

# Gets group lines then removes user if exist.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | while read line
    do
        if [ ! -z "`echo $line | $SED -n "s/^\($groupname:\)\(.*\)$/\1/p"`" ]; then
            echo $line | $SED "s/\( \|:\)$username\( \|$\)/\1\2/" | \
                 $SED "s/: /:/" | $SED "s/  / /" | $SED "s/ $//" >> $APACHEAUTHDIR/$vhost.group
        else
            echo $line >> $APACHEAUTHDIR/$vhost.group
        fi
    done
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    log $MSG_WSM_DELUSERFROMGROUP_DELETED $vhost $username $groupname
    return $EXIT_SUCC
}

# wsm_configure
# Graceful setup or reconfiguration of wsm2 environment.
function wsm_configure {

# Checking components and circumstances.
    if [ ! $UID -eq 0 ]; then echo -e "$MSG_WSM_ALL_ROOTNEED" >&2; return $EXIT_ERR; fi

# Logs handling: wsm2 log creation (rotated via Apache cronjob).
# Must be the 1st step to enable logging configuration process itself.
    if [ ! -f "$WSMLOGFILE" ]; then
        touch $WSMLOGFILE; chown root:root $WSMLOGFILE; chmod 640 $WSMLOGFILE
        log "$MSG_WSM_CONF_LOGFILE_CREATED" "$WSMLOGFILE"
    fi

# Creating timestamp to name backup files and logfile.
    timestamp="$(date '+%Y%m%d%H%M%S')"

# Configuration itself.
    log $($APACHEDAEMON stop 2>/dev/null)

    log $MSG_WSM_CONF_GLOBAL
    # Hardening access rights for web storage (others --X).
    chown $APACHEADMIN:$APACHEADMINGROUP $APACHEDOCROOT
    chmod 2751 $APACHEDOCROOT; $SETFACL -d -m o::X $APACHEDOCROOT
    # Removing default frontpage if any.
    rm $APACHEDOCROOT/index.html 2>/dev/null
    # Creating folder for basic authentication files.
    if [ ! -d $APACHEAUTHDIR ]; then log $(mkdir --verbose -m 755 $APACHEAUTHDIR 2>&1); fi
    # Creating multiuser lock directory.
    if [ ! -d $APACHELOCKDIR ]; then
        log $(mkdir --verbose -m 2770 $APACHELOCKDIR 2>&1)
        chown $APACHEUSER:$APACHEGROUP $APACHELOCKDIR
        $SETFACL -d -m g::rwX $APACHELOCKDIR
    fi
    # Modifying access control on log directory (read for web administrator).
    if [ -d $APACHELOGDIR ]; then 
        $SETFACL    -m u:$APACHEADMIN:rX $APACHELOGDIR
        $SETFACL -d -m u:$APACHEADMIN:r  $APACHELOGDIR
    fi
    # wsm2 global configuration becomes a symlink, local becomes a copy.
    if [ ! -f $APACHECONFIGDIR/$WSMCONFIGFILE ]; then
        log $(ln -v -s $WSMTEMPLATEDIR/$WSMCONFIGFILE $APACHECONFIGDIR/$WSMCONFIGFILE)
    fi
    # Copying wsm2 local configuration template.
    if [ ! -f $APACHECONFIGDIR/$WSMCONFIGLOCAL ]; then
        log $(cp -v -p $WSMTEMPLATEDIR/$WSMCONFIGLOCAL $APACHECONFIGDIR/$WSMCONFIGLOCAL)
    fi
    # Copying urlcheck templates.
    if [ ! -f "$URLCHECK_EXC" ]; then 
        log $(cp -v -p $WSMTEMPLATEDIR/$($BASENAME $URLCHECK_EXC) $URLCHECK_EXC)
    fi
    if [ ! -f "$URLCHECK_HOSTS" ]; then 
        log $(cp -v -p $WSMTEMPLATEDIR/$($BASENAME $URLCHECK_HOSTS) $URLCHECK_HOSTS)
    fi

    # Setting modules.
    log $MSG_WSM_CONF_MODULES
    log $($APACHEDISMOD cgi)
    log $($APACHEENMOD dav_fs expires geoip headers mod-security proxy_http rewrite ssl)

    # Setting ModSecurity.
    log $MSG_WSM_CONF_MODSEC
    # Creating multiuser workspace folders.
    if [ ! -d $WSMMODSECDIR ]; then
        log $(mkdir --verbose -m 2770 $WSMMODSECDIR 2>&1)
        chown $APACHEUSER:$APACHEGROUP $WSMMODSECDIR
        $SETFACL -d -m g::rwX $WSMMODSECDIR
        for folder in $WSMMODSECDIRS
        do
            if [ ! -d "$WSMMODSECDIR/$folder" ]; then
                log $(mkdir --verbose -m 2770 "$WSMMODSECDIR/$folder" 2>&1)
                chown $APACHEUSER:$APACHEGROUP "$WSMMODSECDIR/$folder"
            fi
        done
    fi
    # Copying local configuration template.
    if [ ! -f $APACHECONFIGDIR/$WSMMODSECLOCAL ]; then
        log $(cp -v -p $WSMTEMPLATEDIR/$WSMMODSECLOCAL $APACHECONFIGDIR/$WSMMODSECLOCAL)
    fi
    # Activating builtin ruleset.
    for config in $WSMMODSECCONFS
    do
	if [ ! -L $WSMMODSECCONFDIR/$config ]; then
	    log $(ln -v -s $config.dpkg $WSMMODSECCONFDIR/$config)
	fi
    done
    # Setting AwStats.
    log $MSG_WSM_CONF_AWSTATS
    # Symlinking necessary Perl scripts.
    for plfile in $AWSTATSPROGS
    do
	# Try maintainer's examples (Lenny way).
        if [ ! -L $AWSTATSBINDIR/$plfile -a -f $AWSTATSEXAMPLEDIR/$plfile ]; then
            log $(ln -v -s $AWSTATSEXAMPLEDIR/$plfile $AWSTATSBINDIR/$plfile)
        fi
	# Try maintainer's tools (Squeeze way).
        if [ ! -L $AWSTATSBINDIR/$plfile -a -f $AWSTATSTOOLSDIR/$plfile ]; then
            log $(ln -v -s $AWSTATSTOOLSDIR/$plfile $AWSTATSBINDIR/$plfile)
        fi
    done
    # Moving GeoIPFree database to R/W partition.
    if [ ! -f $GEOIPDATABASE ]; then
        log $(mv -v $GEOIPSOURCE $GEOIPDATABASE)
        log $(ln -v -s $GEOIPDATABASE $GEOIPSOURCE)
    fi
    # Moving GeoIP Lite Country database to R/W partition.
    if [ ! -f $GEOLITEDATABASE ]; then
        log $(mv -v $GEOLITESOURCE $GEOLITEDATABASE)
        log $(ln -v -s $GEOLITEDATABASE $GEOLITESOURCE)
    fi
    # Copying AwStats templates.
    if [ ! -f "$AWSTATS_EXC" ]; then 
        log $(cp -v -p $WSMTEMPLATEDIR/$($BASENAME "$AWSTATS_EXC") "$AWSTATS_EXC")
    fi
    if [ ! -f "$AWSTATS_HOSTS" ]; then 
        log $(cp -v -p $WSMTEMPLATEDIR/$($BASENAME "$AWSTATS_HOSTS") "$AWSTATS_HOSTS")
    fi
    # Creating AwStats result HTML directory.
    if [ ! -d $AWSTATSWEBROOT ]; then
        log $(mkdir --verbose -m 2750 $AWSTATSWEBROOT 2>&1)
        chown $APACHEADMIN:$APACHEADMINGROUP $AWSTATSWEBROOT
    fi

    # Setting PHP (if any).
    log $MSG_WSM_CONF_PHP5
    if [ -d "$PHPCONFDIR" ]; then
	if [ ! -f "$PHPCONFDIR/$PHPCONFFILE" ]; then
	    log $(cp -v -p "$WSMTEMPLATEDIR/$PHPCONFFILE" "$PHPCONFDIR/$PHPCONFFILE")
	fi
    fi

    # Symlinking SSL certificate.
    if [ ! -e "$SSLCONFDIR/certs/$DEFAULT_HOSTNAME.pem" ]; then
        log "$MSG_WSM_CONF_DEFAULT_CERT"
        ln -s "$SSLCONFDIR/certs/ssl-cert-snakeoil.pem" "$SSLCONFDIR/certs/$DEFAULT_HOSTNAME.pem"
        log "$SSLCONFDIR/certs/$DEFAULT_HOSTNAME.pem" "-> " "$SSLCONFDIR/certs/ssl-cert-snakeoil.pem"
        rm -f "$SSLCONFDIR/private/$DEFAULT_HOSTNAME.key" 2>/dev/null
        ln -s "$SSLCONFDIR/private/ssl-cert-snakeoil.key" "$SSLCONFDIR/private/$DEFAULT_HOSTNAME.key"
        log "$SSLCONFDIR/private/$DEFAULT_HOSTNAME.key" "->" "$SSLCONFDIR/private/ssl-cert-snakeoil.key"
    else
        log "$MSG_WSM_CONF_DEFAULT_CERT_EXISTS"
    fi

    # Creating default virtualhost.
    if [ -L "$APACHEVHOSTSDIR/$DEFAULT_VHOST" ]; then
        # Normally default configuration is a regular file;
        # when symlink found, it is created by wsm2 and nothing to do.
        log $MSG_WSM_CONF_DEFAULT_VHOST_EXISTS
    else
        log $MSG_WSM_CONF_DEFAULT_VHOST
        # Creating a new virtualhost via recursive wsm2 call.
        $0 -cw $DEFAULT_HOSTNAME $DEFAULT_EMAIL
        # Activating new virtualhost as default.
        log $($APACHEDISSITE default)
        log $($APACHEDISSITE $DEFAULT_HOSTNAME)
        # Backing up exisitng default and symlinking just created.
        mv $APACHEVHOSTSDIR/$DEFAULT_VHOST $APACHEVHOSTSDIR/$DEFAULT_VHOST.$timestamp
        ln -s $APACHEVHOSTSDIR/$DEFAULT_HOSTNAME $APACHEVHOSTSDIR/$DEFAULT_VHOST
        log $($APACHEENSITE default)
    fi

# Finally go on with Apache!
    log $($APACHEDAEMON start)
    return
}

# Helper function: logs with timestamp into a logfile and native to STD_ERR.
function log {

    local DATETIME="+%d/%b/%Y:%H:%M:%S %Z"
    if [ -f "$WSMLOGFILE" -a -w "$WSMLOGFILE" ]; then
        echo -e "[$($DATE "$DATETIME")] $($WHOAMI) ${FUNCNAME[1]} $*" >>$WSMLOGFILE
        echo -e "$*" >&2
    else
        echo -e "[$($DATE "$DATETIME")] $($WHOAMI) ${FUNCNAME[1]} $*" >&2
    fi
    return
}

#
# Main program - handling options
#

option=`echo $1 | $SED -n "s/^-\+//p"`; if [ ! -z "$option" ]; then shift; fi
case "$option" in
    "cw"|"createweb")
        wsm_createweb $*; exit $?
    ;;
    "rw"|"removeweb")
        wsm_removeweb $*; exit $?
    ;;
    "sw"|"saveweb")
        wsm_saveweb $*; exit $?
    ;;
    "au"|"adduser")
        wsm_adduser $*; exit $?
    ;;
    "du"|"deluser")
        wsm_deluser $*; exit $?
    ;;
    "ag"|"addgroup")
        wsm_addgroup $*; exit $?
    ;;
    "dg"|"delgroup")
        wsm_delgroup $*; exit $?
    ;;
    "aug"|"addusertogroup")
        wsm_addusertogroup $*; exit $?
    ;;
    "dug"|"deluserfromgroup")
        wsm_deluserfromgroup $*; exit $?
    ;;
    "lug"|"listusersgroup")
        wsm_listusersgroup $*; exit $?
    ;;
    "configure"|"install")
        wsm_configure $*; exit $?
    ;;
    *)
        echo -e "$USAGE"; exit $EXIT_ERR
    ;;
esac
